/* 
 * File:   Timer.cpp
 * Author: martin
 * 
 * Created on May 5, 2010, 1:29 PM
 */
#include<ctime>

#include "Timer.h"
using std::clock_t;
using std::clock;
#define _TIMER_HIGH_PRECISION_

Timer::Timer() {
	started = false;
}

Timer::~Timer() {
}

void Timer::start(){
	if ( started )
		throw already_started_exception();
	started = true;
	start_time = clock();
	last_lap_started_at = start_time;
}

void Timer::stop(){
	if ( ! started )
		throw not_started_exception();
	started = false;
	stop_time = clock();
}

double Timer::get_duration() const{
	if ( started )
		throw already_started_exception();
	return double(stop_time - start_time )/ CLOCKS_PER_SEC;
}

double Timer::get_lap() const{
	return double(clock() - last_lap_started_at )/ CLOCKS_PER_SEC;
}

double Timer::get_last_lap()
{
	int started_at = last_lap_started_at;
	last_lap_started_at = clock();
	return double(last_lap_started_at - started_at)/ CLOCKS_PER_SEC;
}

void Timer::pause(){
	if ( ! started )
		throw not_started_exception();
	if ( ! paused )
		paused_at = clock();
	paused = true;
}

void Timer::unpause(){
	if ( ! started )
		throw not_started_exception();
	if ( ! paused )
		return;
	paused = false;
	clock_t diff = clock() - paused_at;
	start_time += diff;
	last_lap_started_at += diff;
}

clock_t Timer::clock() const{
#ifdef _TIMER_HIGH_PRECISION_
	struct timespec ts;
	clock_gettime( CLOCK_REALTIME, &ts);
	const time_t nano_per_clock = 1E9/CLOCKS_PER_SEC;
	clock_t ct = ( ts.tv_sec * CLOCKS_PER_SEC ) + (ts.tv_nsec / nano_per_clock   
												  );
	return ct;
#else
	return std::clock();
#endif

}
