/* 
 * File:   Timer.h
 * Author: martin
 *
 * Created on May 5, 2010, 1:29 PM
 * Define _TIMER_HIGH_PRECISION_  to use POSIX timespec and clock_gettime(3)
 * Measured time will still use CLOCKS_PER_SEC  (POSIX: 1000000)
 */

#ifndef _TIMER_H
#define	_TIMER_H
#include<ctime>

// Uncomment to activate. Might not work on all compilers.
// If c++11 is available, you can also use std::chrono timers.
// #define _TIMER_HIGH_PRECISION_

class Timer {
	public:
		Timer();
		virtual ~Timer();
		void start();
		void stop();
		void pause();
		void unpause();
		double get_duration() const;
		double get_lap() const;
		double get_last_lap();
		struct not_started_exception {};
		struct already_started_exception {};
	private:
		clock_t clock() const;
		std::clock_t start_time;
		std::clock_t stop_time;
		std::clock_t last_lap_started_at;
		bool started;
		std::clock_t paused_at;
		bool paused;
};

#endif	/* _TIMER_H */

